from .lib_llm_inference import (
    InvoiceToChatGPT3_5,
    InvoiceToHFLlama13B,
)

from .lib_pdf_manipulation import (
    ConvertPdfToImages,
    ExtractTextFromPdf, 
    ExtractTextFromImgs, 
)

from .lib_settings import (
  FineTuningSettings,
  Config,
)


## Project Level Constants 

VERSION = 1 

class BasicInferenceRequest:
  def __init__(self, token="", inference_url="", system_prompt="", content_prompt="", output_prompt=""):
    self.token = token
    self.inference_url = inference_url
    self.system_prompt = system_prompt
    self.content_prompt = content_prompt
    self.output_prompt = output_prompt


class ArtifactNames:
  def __init__(self, base_model, new_model_name, repo_name, dataset):
    self.base_model = base_model
    base_model_name = base_model.split("/")[-1]
    self.adapter_model = repo_name + "/" + base_model_name + "_" + new_model_name
    self.new_model = repo_name + "/" + new_model_name 
    self.dataset = dataset 


# Library of Prompt related functions for various types of models

def debugPrintMessage(prompt, message = ""):
  print(" == debug " + message + " ====================")
  print(prompt)
  print(" ====================================")


def createLlama2Prompt(system_prompt: str, content_prompt: str, output_prompt: str, debug = False) -> str:
  prompt = f"""
<s>[INST] <<SYS>>
{system_prompt}
<</SYS>>

{content_prompt} {output_prompt}[/INST]
""".strip()

  if (debug == True):
    debugPrintMessage(prompt, "prompt")

  return prompt


def createOpenAIPrompt(system_prompt: str, content_prompt: str, debug = False) -> str:
  prompt = f"""
  {system_prompt} 

  {content_prompt}
  """.strip()

  if (debug == True):
    debugPrintMessage(prompt, "prompt")

  return prompt




from langchain.chat_models import ChatOpenAI
from langchain.prompts import PromptTemplate
from langchain.chains import LLMChain
from huggingface_hub import InferenceClient

def InvoiceToChatGPT3_5(content: str, template: str, data_points):
  llm = ChatOpenAI(temperature=0, model="gpt-3.5-turbo-0613")

  if template == "":
    template = """
    You are an expert admin who will extract core information from documents 

    {content}

    Above is the content; please try to extract all data points from the content above
    and export in a JSON array format for each invoice line item:
    {data_points}

    Now please extract details from the content and export in a JSON array format,
    return ONLY the JSON array:
    """

  prompt = PromptTemplate(
    input_variables=["content", "data_points"],
    template=template,
  )

  chain = LLMChain(llm=llm, prompt=prompt)

  results = chain.run(content=content, data_points=data_points)

  return results

def InvoiceToHFLlama13B(content: str, template: str, data_points, hf_token, endpoint_url):

  client = InferenceClient(token=hf_token, model=endpoint_url)

  # generation parameter
  gen_kwargs = dict(
    max_new_tokens=748,
    top_k=30,
    top_p=0.9,
    temperature=0.2,
    repetition_penalty=1.02,
    stream=False,
    details=True,
  )

  if template == "":
    template = """
    You are an expert admin who will extract core information from documents 

    {content}

    Above is the content; please try to extract all data points from the content above
    and export in a JSON array format:
    {data_points}

    """

  prompt = PromptTemplate.from_template(template)
  prompt_str = prompt.format(content=content, data_points=data_points)

  print(prompt)
  print("================================")
  print(prompt_str)
  print("================================")
  # prompt = "What killed the dinosaurs?"
  response = client.text_generation(prompt_str, **gen_kwargs)
  print(response.generated_text)

  return response.generated_text 